import ac
import acsys
import os
import sys
import platform
import configparser
import math

if platform.architecture()[0] == "64bit":
    libdir = 'third_party/lib64'
else:
    libdir = 'third_party/lib'
sys.path.insert(0, os.path.join(os.path.dirname(__file__), libdir))
os.environ['PATH'] = os.environ['PATH'] + ";."

from third_party.sim_info import info

app_path = "apps/python/IDAS6hud/"

# Assume config doesn't need to be updated
update_config = False

# Set config file path
config_path = app_path + "config.ini"

# Initialize configparser
config = configparser.ConfigParser()
config.read(config_path)

# Check if config file has appropriate content
if config.has_section("IDAS6hud") != True:
    config.add_section("IDAS6hud")
    update_config = True


# Load the boolean value or set it to default
def get_boolean(config, key, default):
    global update_config
    try:
        return config.getboolean("IDAS6hud", key)
    except:
        config.set("IDAS6hud", key, str(default))
        update_config = True
        return default

# Load the float value or set it to default
def get_float(config, key, default):
    global update_config
    try:
        return config.getfloat("IDAS6hud", key)
    except:
        config.set("IDAS6hud", key, str(default))
        update_config = True
        return default

# Load the integer value or set it to default
def get_int(config, key, default):
    global update_config
    try:
        return config.getint("IDAS6hud", key)
    except:
        config.set("IDAS6hud", key, str(default))
        update_config = True
        return default

# Load or generate customizable parameters
theme = get_int(config, "theme", 6)
scale = get_float(config, "scale", 100.0) / 100
unit_kmh = get_boolean(config, "unit_kmh", True)
show_drift = get_boolean(config, "show_drift", True)
show_pedal = get_boolean(config, "show_pedal", False)
lower_refresh_rate = get_boolean(config, "lower_refresh_rate", False)
night_mode = get_boolean(config, "night_mode", False)
unlimited_name = get_boolean(config, "unlimited_name", False)

# Update config if necessary
if update_config:
    with open(config_path, 'w') as file_config:
        config.write(file_config)

# Timers
timer = 0
timer2 = 0
timer3 = 0
alpha = -0.1

# Window visibilities
settings_window_visibility = 0

# Set image paths
theme_path = "themes/D" + str(theme) + "/"

theme_config = app_path + theme_path + "theme_config.ini"
config.read(theme_config)

custom_font = config.get("Font", "custom_font")
italic = config.getint("Font", "italic")
bold = config.getint("Font", "bold")

tacho_width = config.getfloat("Tachometer", "tacho_width")
tacho_height = config.getfloat("Tachometer", "tacho_height")
tachowithPedal_width = config.getfloat("Tachometer", "tachowithPedal_width")
tachowithPedal_height = config.getfloat("Tachometer", "tachowithPedal_height")
gear_x = config.getfloat("Tachometer", "gear_x")
gear_y = config.getfloat("Tachometer", "gear_y")
gear_width = config.getfloat("Tachometer", "gear_width")
gear_height = config.getfloat("Tachometer", "gear_height")
gauge_x = config.getfloat("Tachometer", "gauge_x")
gauge_y = config.getfloat("Tachometer", "gauge_y")
gauge_width = config.getfloat("Tachometer", "gauge_width")
gauge_height = config.getfloat("Tachometer", "gauge_height")
rpm_x = config.getfloat("Tachometer", "rpm_x")
rpm_y = config.getfloat("Tachometer", "rpm_y")
rpm_width = config.getfloat("Tachometer", "rpm_width")
rpm_height = config.getfloat("Tachometer", "rpm_height")
speed_x = config.getfloat("Tachometer", "speed_x")
speed_y = config.getfloat("Tachometer", "speed_y")
speed_width = config.getfloat("Tachometer", "speed_width")
speed_height = config.getfloat("Tachometer", "speed_height")
unit_x = config.getfloat("Tachometer", "unit_x")
unit_y = config.getfloat("Tachometer", "unit_y")
unit_width = config.getfloat("Tachometer", "unit_width")
unit_height = config.getfloat("Tachometer", "unit_height")

center_x = config.getfloat("Pedal", "center_x")
center_y = config.getfloat("Pedal", "center_y")
out_radius_gas = config.getfloat("Pedal", "out_radius_gas")
in_radius_gas = config.getfloat("Pedal", "in_radius_gas")
out_radius_brake = config.getfloat("Pedal", "out_radius_brake")
in_radius_brake = config.getfloat("Pedal", "in_radius_brake")
degree_gas = config.getfloat("Pedal", "degree_gas")
degree_brake = config.getfloat("Pedal", "degree_brake")
offset = config.getfloat("Pedal", "offset")

if theme != 5:
    drift_x = config.getfloat("Drift Light", "drift_x")
    drift_y = config.getfloat("Drift Light", "drift_y")
    drift_width = config.getfloat("Drift Light", "drift_width")
    drift_height = config.getfloat("Drift Light", "drift_height")

map_width = config.getfloat("Map", "map_width")
map_height = config.getfloat("Map", "map_height")
lapCount_x = config.getfloat("Map", "lapCount_x")
lapCount_y = config.getfloat("Map", "lapCount_y")
lapCount_fontSize = config.getfloat("Map", "lapCount_fontSize")
lapCount_color_R = config.getfloat("Map", "lapCount_color_R")
lapCount_color_G = config.getfloat("Map", "lapCount_color_G")
lapCount_color_B = config.getfloat("Map", "lapCount_color_B")
lapCount_color_Alpha = config.getfloat("Map", "lapCount_color_Alpha")
trackName_x = config.getfloat("Map", "trackName_x")
trackName_y = config.getfloat("Map", "trackName_y")
trackName_width = config.getfloat("Map", "trackName_width")
trackName_height = config.getfloat("Map", "trackName_height")
trackConfig_x = config.getfloat("Map", "trackConfig_x")
trackConfig_y = config.getfloat("Map", "trackConfig_y")
trackConfig_width = config.getfloat("Map", "trackConfig_width")
trackConfig_height = config.getfloat("Map", "trackConfig_height")

time_width = config.getfloat("Lap Time", "time_width")
time_height = config.getfloat("Lap Time", "time_height")
currentLap_x = config.getfloat("Lap Time", "currentLap_x")
currentLap_y = config.getfloat("Lap Time", "currentLap_y")
currentLap_fontSize = config.getfloat("Lap Time", "currentLap_fontSize")
currentLap_color_R = config.getfloat("Lap Time", "currentLap_color_R")
currentLap_color_G = config.getfloat("Lap Time", "currentLap_color_G")
currentLap_color_B = config.getfloat("Lap Time", "currentLap_color_B")
currentLap_color_Alpha = config.getfloat("Lap Time", "currentLap_color_Alpha")

record_width = config.getfloat("Records", "record_width")
record_height = config.getfloat("Records", "record_height")
bestLap_x = config.getfloat("Records", "bestLap_x")
bestLap_y = config.getfloat("Records", "bestLap_y")
bestLap_fontSize = config.getfloat("Records", "bestLap_fontSize")
bestLap_color_R = config.getfloat("Records", "bestLap_color_R")
bestLap_color_G = config.getfloat("Records", "bestLap_color_G")
bestLap_color_B = config.getfloat("Records", "bestLap_color_B")
bestLap_color_Alpha = config.getfloat("Records", "bestLap_color_Alpha")
lastLap_x = config.getfloat("Records", "lastLap_x")
lastLap_y = config.getfloat("Records", "lastLap_y")
lastLap_fontSize = config.getfloat("Records", "lastLap_fontSize")
lastLap_color_R = config.getfloat("Records", "lastLap_color_R")
lastLap_color_G = config.getfloat("Records", "lastLap_color_G")
lastLap_color_B = config.getfloat("Records", "lastLap_color_B")
lastLap_color_Alpha = config.getfloat("Records", "lastLap_color_Alpha")

name_width = config.getfloat("Name", "name_width")
name_height = config.getfloat("Name", "name_height")
driver_x = config.getfloat("Name", "driver_x")
driver_y = config.getfloat("Name", "driver_y")
driver_fontSize = config.getfloat("Name", "driver_fontSize")
driver_color_R = config.getfloat("Name", "driver_color_R")
driver_color_G = config.getfloat("Name", "driver_color_G")
driver_color_B = config.getfloat("Name", "driver_color_B")
driver_color_Alpha = config.getfloat("Name", "driver_color_Alpha")
car_x = config.getfloat("Name", "car_x")
car_y = config.getfloat("Name", "car_y")
car_fontSize = config.getfloat("Name", "car_fontSize")
car_fontAlignment = config.get("Name", "car_fontAlignment")
car_color_R = config.getfloat("Name", "car_color_R")
car_color_G = config.getfloat("Name", "car_color_G")
car_color_B = config.getfloat("Name", "car_color_B")
car_color_Alpha = config.getfloat("Name", "car_color_Alpha")
position_x = config.getfloat("Name", "position_x")
position_y = config.getfloat("Name", "position_y")
position_width = config.getfloat("Name", "position_width")
position_height = config.getfloat("Name", "position_height")

background = app_path + theme_path + "background/background.png"
background_pedal = app_path + theme_path + "background/background_pedal.png"
nodrift_background = app_path + theme_path + "background/nodrift_background.png"
nodrift_background_pedal = app_path + theme_path + "background/nodrift_background_pedal.png"
map_background = app_path + theme_path + "background/map_background.png"
time_background = app_path + theme_path + "background/time_background.png"
record_background = app_path + theme_path + "background/record_background.png"
name_background = app_path + theme_path + "background/name_background.png"
kmh = app_path + theme_path + "speed_unit/kmh.png"
mph = app_path + theme_path + "speed_unit/mph.png"
preview = app_path + theme_path + "preview.png"


def acMain(ac_version):
    global app_window, map_window, time_window, record_window, name_window, settings_window
    global gear_label, drift_label, gauge_path, gauge_label, rpm_path, rpm_gauge, speed_path, speed_unit, speed_digits
    global lap_count, track_name, track_config
    global current_lap, last_lap, best_lap
    global driver_label, car_label, position_digits
    global theme_label, theme_spinner
    global scale_label, scale_spinner
    global unit_kmh_label, unit_kmh_checkbox
    global show_drift_label, show_drift_checkbox
    global show_pedal_label, show_pedal_checkbox
    global refresh_rate_label, refresh_rate_checkbox
    global night_label, night_checkbox
    global unlimited_name_label, unlimited_name_checkbox
    global preview_label
    global drift_blue, drift_yellow, rev_light, rev_brown

    if night_mode:
        gauge_path = theme_path + "background/night_labels_"
        rpm_path = theme_path + "rpm_bar/night_rpm_"
    else:
        gauge_path = theme_path + "background/labels_"
        rpm_path = theme_path + "rpm_bar/rpm_"

    if theme != 7:
        speed_path = theme_path + "speed_digits/speed_digits_"

    # Initialize font
    ac.initFont(0, custom_font, italic, bold)

    # App window
    app_window = ac.newApp("D6tacho")
    ac.setTitle(app_window, "")
    ac.drawBorder(app_window, 0)
    ac.setIconPosition(app_window, 0, -10000)

    if show_pedal:
        ac.setSize(app_window, tachowithPedal_width * scale, tachowithPedal_height * scale)
    else:
        ac.setSize(app_window, tacho_width * scale, tacho_height * scale)

    if show_drift:
        if show_pedal:
            ac.setBackgroundTexture(app_window, background_pedal)
        else:
            ac.setBackgroundTexture(app_window, background)
    else:
        if show_pedal:
            ac.setBackgroundTexture(app_window, nodrift_background_pedal)
        else:
            ac.setBackgroundTexture(app_window, nodrift_background)

    ac.setBackgroundOpacity(app_window, 0)

    drift_blue = ac.newTexture(app_path + theme_path + "drift/drift_blue.png")
    drift_yellow = ac.newTexture(app_path + theme_path + "drift/drift_yellow.png")
    rev_light = ac.newTexture(app_path + theme_path + "rev_light.png")
    rev_brown = ac.newTexture(app_path + theme_path + "rev_lightbrown.png")
    ac.addRenderCallback(app_window, pedal)

    # Gear label
    gear_label = ac.addLabel(app_window, "")
    ac.setSize(gear_label, gear_width * scale, gear_height * scale)
    ac.setPosition(gear_label, gear_x * scale, gear_y * scale)

    # Gauge label
    gauge_label = ac.addLabel(app_window, "")
    ac.setSize(gauge_label, gauge_width * scale, gauge_height * scale)
    ac.setPosition(gauge_label, gauge_x * scale, gauge_y * scale)

    # RPM gauge
    rpm_gauge = ac.addLabel(app_window, "")
    ac.setSize(rpm_gauge, rpm_width * scale, rpm_height * scale)
    ac.setPosition(rpm_gauge, rpm_x * scale, rpm_y * scale)

    # Speed digits
    speed_digits = []
    for i in range(3):
        label_width = speed_width * scale
        speed_digits.append(ac.addLabel(app_window, ""))
        ac.setSize(speed_digits[i], label_width, speed_height * scale)
        ac.setPosition(speed_digits[i], speed_x * scale + i * label_width, speed_y * scale)

    # Speed unit
    speed_unit = ac.addLabel(app_window, "")
    ac.setSize(speed_unit, unit_width * scale, unit_height * scale)
    ac.setPosition(speed_unit, unit_x * scale, unit_y * scale)
    if unit_kmh:
        ac.setBackgroundTexture(speed_unit, kmh)
    else:
        ac.setBackgroundTexture(speed_unit, mph)

    # Map window
    map_window = ac.newApp("D6map")
    ac.setTitle(map_window, "")
    ac.drawBorder(map_window, 0)
    ac.setIconPosition(map_window, 0, -10000)
    ac.setSize(map_window, map_width * scale, map_height * scale)
    ac.setBackgroundTexture(map_window, map_background)
    ac.setBackgroundOpacity(map_window, 0)

    # Lap count
    if theme != 5:
        lap_count = ac.addLabel(map_window, "")
        ac.setPosition(lap_count, lapCount_x * scale, lapCount_y * scale)
        ac.setFontSize(lap_count, lapCount_fontSize * scale)
        ac.setCustomFont(lap_count, custom_font, italic, bold)
        ac.setFontColor(lap_count, lapCount_color_R, lapCount_color_G, lapCount_color_B, lapCount_color_Alpha)
        ac.setFontAlignment(lap_count, "center")

    # Track name
    track_name = ac.addLabel(map_window, "")
    ac.setSize(track_name, trackName_width * scale, trackName_height * scale)
    ac.setPosition(track_name, trackName_x * scale, trackName_y * scale)
    track_id = {
        "mount_akina_2017": theme_path + "track_name/akina.png",
        "corsa_club_akagi": theme_path + "track_name/akagi.png",
        "akagi mountain pass": theme_path + "track_name/akagi.png",
        "corsa_club_shomaru_pass": theme_path + "track_name/shomaru.png",
        "shomaru": theme_path + "track_name/shomaru.png",
        "corsa_club_tsuchisaka": theme_path + "track_name/tsuchisaka.png",
        "sost_irohazaka": theme_path + "track_name/irohazaka.png",
        "sost_myogi": theme_path + "track_name/myogi.png",
        "c1": theme_path + "track_name/kanjo.png",
        "c1-midnight_v13": theme_path + "track_name/kanjo.png",
        "wangan": theme_path + "track_name/wangan.png",
        "takigahara": theme_path + "track_name/takigahara.png",
        "pk_usui_pass": theme_path + "track_name/usui.png",
        "hakone_ohiradai_v2": theme_path + "track_name/ohiradai.png",
        "hakone_nanamagari_v2": theme_path + "track_name/nanamagari.png",
        "nikkocircuit": theme_path + "track_name/nikko.png",
        "suzuka": theme_path + "track_name/suzuka.png",
        "nanohanadai": theme_path + "track_name/nanohanadai.png",
        "tsukuba_circuit": theme_path + "track_name/tsukuba.png",
        "lake_haruna": theme_path + "track_name/akina_lake.png",
        "shoubudani": theme_path + "track_name/shoubudani.png",
        "ww_metropolitan_expressway": theme_path + "track_name/shutokou.png"
    }
    trackName = ac.getTrackName(0)
    if trackName in track_id:
        ac.setBackgroundTexture(track_name, app_path + track_id[trackName])

    # Track config
    if theme != 5:
        track_config = ac.addLabel(map_window, "")
        ac.setSize(track_config, trackConfig_width * scale, trackConfig_height * scale)
        ac.setPosition(track_config, trackConfig_x * scale, trackConfig_y * scale)
        config_id = {
            "layout_downhill": theme_path + "track_config/downhill.png",
            "down": theme_path + "track_config/downhill.png",
            "usuipass_short_dh": theme_path + "track_config/downhill.png",
            "haruna_downhill": theme_path + "track_config/downhill.png",
            "layout_uphill": theme_path + "track_config/uphill.png",
            "up": theme_path + "track_config/uphill.png",
            "shomaru_inbound": theme_path + "track_config/inbound.png",
            "shomaru_outbound": theme_path + "track_config/outbound.png",
            "normal": theme_path + "track_config/outbound.png",
            "reverse": theme_path + "track_config/inbound.png",
            "c1_in": theme_path + "track_config/c1_in.png",
            "c1_out": theme_path + "track_config/c1_out.png",
            "c1": theme_path + "track_config/c1_out.png"
        }
        trackConfig = ac.getTrackConfiguration(0)
        if trackConfig in config_id:
            ac.setBackgroundTexture(track_config, app_path + config_id[trackConfig])

    # Time window
    time_window = ac.newApp("D6time")
    ac.setTitle(time_window, "")
    ac.drawBorder(time_window, 0)
    ac.setIconPosition(time_window, 0, -10000)
    ac.setSize(time_window, time_width * scale, time_height * scale)
    ac.setBackgroundTexture(time_window, time_background)
    ac.setBackgroundOpacity(time_window, 0)

    if theme == 5:
        lap_count = ac.addLabel(time_window, "")
        ac.setPosition(lap_count, lapCount_x * scale, lapCount_y * scale)
        ac.setFontSize(lap_count, lapCount_fontSize * scale)
        ac.setCustomFont(lap_count, custom_font, italic, bold)
        ac.setFontColor(lap_count, lapCount_color_R, lapCount_color_G, lapCount_color_B, lapCount_color_Alpha)
        ac.setFontAlignment(lap_count, "center")

    # Current lap
    current_lap = ac.addLabel(time_window, "")
    ac.setPosition(current_lap, currentLap_x * scale, currentLap_y * scale)
    ac.setFontSize(current_lap, currentLap_fontSize * scale)
    ac.setCustomFont(current_lap, custom_font, italic, bold)
    ac.setFontColor(current_lap, currentLap_color_R, currentLap_color_G, currentLap_color_B, currentLap_color_Alpha)
    ac.setFontAlignment(current_lap, "center")

    # Record window
    record_window = ac.newApp("D6record")
    ac.setTitle(record_window, "")
    ac.drawBorder(record_window, 0)
    ac.setIconPosition(record_window, 0, -10000)
    ac.setSize(record_window, record_width * scale, record_height * scale)
    ac.setBackgroundTexture(record_window, record_background)
    ac.setBackgroundOpacity(record_window, 0)

    # Best lap
    best_lap = ac.addLabel(record_window, "")
    ac.setPosition(best_lap, bestLap_x * scale, bestLap_y * scale)
    ac.setFontSize(best_lap, bestLap_fontSize * scale)
    ac.setCustomFont(best_lap, custom_font, italic, bold)
    ac.setFontColor(best_lap, bestLap_color_R, bestLap_color_G, bestLap_color_B, bestLap_color_Alpha)
    ac.setFontAlignment(best_lap, "center")

    # Last lap
    last_lap = ac.addLabel(record_window, "")
    ac.setPosition(last_lap, lastLap_x * scale, lastLap_y * scale)
    ac.setFontSize(last_lap, lastLap_fontSize * scale)
    ac.setCustomFont(last_lap, custom_font, italic, bold)
    ac.setFontColor(last_lap, lastLap_color_R, lastLap_color_G, lastLap_color_B, lastLap_color_Alpha)
    ac.setFontAlignment(last_lap, "center")

    # Name window
    name_window = ac.newApp("D6name")
    ac.setTitle(name_window, "")
    ac.drawBorder(name_window, 0)
    ac.setIconPosition(name_window, 0, -10000)
    ac.setSize(name_window, name_width * scale, name_height * scale)
    ac.setBackgroundTexture(name_window, name_background)
    ac.setBackgroundOpacity(name_window, 0)

    # Driver name
    driver_label = ac.addLabel(name_window, "")
    ac.setPosition(driver_label, driver_x * scale, driver_y * scale)
    ac.setFontSize(driver_label, driver_fontSize * scale)
    # ac.setCustomFont(driver_label, custom_font, italic, bold)
    ac.setFontColor(driver_label, driver_color_R, driver_color_G, driver_color_B, driver_color_Alpha)
    ac.setFontAlignment(driver_label, "center")

    # Car name
    car_label = ac.addLabel(name_window, "")
    ac.setPosition(car_label, car_x * scale, car_y * scale)
    ac.setFontSize(car_label, car_fontSize * scale)
    # ac.setCustomFont(car_label, custom_font, italic, bold)
    ac.setFontColor(car_label, car_color_R, car_color_G, car_color_B, car_color_Alpha)
    ac.setFontAlignment(car_label, car_fontAlignment)

    # Position digits
    position_digits = []
    for i in range(2):
        label_width = position_width * scale
        position_digits.append(ac.addLabel(name_window, ""))
        ac.setSize(position_digits[i], label_width, position_height * scale)
        ac.setPosition(position_digits[i], position_x * scale + i * label_width, position_y * scale)

    # Settings window
    settings_window = ac.newApp("D6settings")
    ac.drawBorder(settings_window, 0)
    ac.setSize(settings_window, 360 * scale, 500 * scale)
    ac.setVisible(settings_window, settings_window_visibility)
    ac.addOnAppActivatedListener(settings_window, settings_window_activated)
    ac.addOnAppDismissedListener(settings_window, settings_window_deactivated)

    # Theme
    theme_label = ac.addLabel(settings_window, "Theme (requires restart)")
    ac.setPosition(theme_label, 10 * scale, 40 * scale)
    ac.setFontSize(theme_label, 20 * scale)

    theme_spinner = ac.addSpinner(settings_window, "")
    ac.setRange(theme_spinner, 5, 7)
    ac.setStep(theme_spinner, 1)
    ac.setValue(theme_spinner, theme)
    ac.setPosition(theme_spinner, 260 * scale, 40 * scale)
    ac.setSize(theme_spinner, 90 * scale, 25 * scale)
    ac.setFontSize(theme_spinner, 20 * scale)
    ac.addOnValueChangeListener(theme_spinner, theme_spinner_clicked)

    # Scale
    scale_label = ac.addLabel(settings_window, "Scale (requires restart)")
    ac.setPosition(scale_label, 10 * scale, 80 * scale)
    ac.setFontSize(scale_label, 20 * scale)

    scale_spinner = ac.addSpinner(settings_window, "")
    ac.setRange(scale_spinner, 50, 200)
    ac.setStep(scale_spinner, 10)
    ac.setValue(scale_spinner, scale * 100)
    ac.setPosition(scale_spinner, 260 * scale, 80 * scale)
    ac.setSize(scale_spinner, 90 * scale, 25 * scale)
    ac.setFontSize(scale_spinner, 20 * scale)
    ac.addOnValueChangeListener(scale_spinner, scale_spinner_clicked)

    # Speed unit
    unit_kmh_label = ac.addLabel(settings_window, "Speed in km/h")
    ac.setPosition(unit_kmh_label, 10 * scale, 120 * scale)
    ac.setFontSize(unit_kmh_label, 20 * scale)

    unit_kmh_checkbox = ac.addCheckBox(settings_window, "")
    ac.setValue(unit_kmh_checkbox, unit_kmh)
    ac.setPosition(unit_kmh_checkbox, 330 * scale, 120 * scale)
    ac.setSize(unit_kmh_checkbox, 20 * scale, 20 * scale)
    ac.addOnCheckBoxChanged(unit_kmh_checkbox, unit_kmh_checkbox_clicked)

    # Drift light
    show_drift_label = ac.addLabel(settings_window, "Show drift light")
    ac.setPosition(show_drift_label, 10 * scale, 160 * scale)
    ac.setFontSize(show_drift_label, 20 * scale)

    show_drift_checkbox = ac.addCheckBox(settings_window, "")
    ac.setValue(show_drift_checkbox, show_drift)
    ac.setPosition(show_drift_checkbox, 330 * scale, 160 * scale)
    ac.setSize(show_drift_checkbox, 20 * scale, 20 * scale)
    ac.addOnCheckBoxChanged(show_drift_checkbox, show_drift_checkbox_clicked)

    # Pedal gauge
    show_pedal_label = ac.addLabel(settings_window, "Show pedal gauge")
    ac.setPosition(show_pedal_label, 10 * scale, 200 * scale)
    ac.setFontSize(show_pedal_label, 20 * scale)

    show_pedal_checkbox = ac.addCheckBox(settings_window, "")
    ac.setValue(show_pedal_checkbox, show_pedal)
    ac.setPosition(show_pedal_checkbox, 330 * scale, 200 * scale)
    ac.setSize(show_pedal_checkbox, 20 * scale, 20 * scale)
    ac.addOnCheckBoxChanged(show_pedal_checkbox, show_pedal_checkbox_clicked)

    # Night mode
    night_label = ac.addLabel(settings_window, "Night mode")
    ac.setPosition(night_label, 10 * scale, 240 * scale)
    ac.setFontSize(night_label, 20 * scale)

    night_checkbox = ac.addCheckBox(settings_window, "")
    ac.setValue(night_checkbox, night_mode)
    ac.setPosition(night_checkbox, 330 * scale, 240 * scale)
    ac.setSize(night_checkbox, 20 * scale, 20 * scale)
    ac.addOnCheckBoxChanged(night_checkbox, night_checkbox_clicked)

    # Lower refresh rate
    refresh_rate_label = ac.addLabel(settings_window, "Lower refresh rate")
    ac.setPosition(refresh_rate_label, 10 * scale, 280 * scale)
    ac.setFontSize(refresh_rate_label, 20 * scale)

    refresh_rate_checkbox = ac.addCheckBox(settings_window, "")
    ac.setValue(refresh_rate_checkbox, lower_refresh_rate)
    ac.setPosition(refresh_rate_checkbox, 330 * scale, 280 * scale)
    ac.setSize(refresh_rate_checkbox, 20 * scale, 20 * scale)
    ac.addOnCheckBoxChanged(refresh_rate_checkbox, refresh_rate_checkbox_clicked)

    # Unlimited driver name's length
    unlimited_name_label = ac.addLabel(settings_window, "Unlimited driver name's length")
    ac.setPosition(unlimited_name_label, 10 * scale, 320 * scale)
    ac.setFontSize(unlimited_name_label, 20 * scale)

    unlimited_name_checkbox = ac.addCheckBox(settings_window, "")
    ac.setValue(unlimited_name_checkbox, lower_refresh_rate)
    ac.setPosition(unlimited_name_checkbox, 330 * scale, 320 * scale)
    ac.setSize(unlimited_name_checkbox, 20 * scale, 20 * scale)
    ac.addOnCheckBoxChanged(unlimited_name_checkbox, unlimited_name_checkbox_clicked)

    # Preview
    preview_label = ac.addLabel(settings_window, "")
    ac.setPosition(preview_label, 0 * scale, 356 * scale)
    ac.setSize(preview_label, 360 * scale, 144 * scale)
    ac.setBackgroundTexture(preview_label, preview)


def pedal(deltaT):
    if show_pedal:
        degreeGas = round(gas * degree_gas)
        degreeBrake = round(brake * degree_brake)

        ac.glBegin(1)
        for i in range(0, degreeGas, 5):
            ac.glBegin(3)
            ac.glColor4f(0, 0.7, 1, 0.75)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset + 5)) * out_radius_gas * scale,
                          center_y * scale - math.cos(math.radians(i + offset + 5)) * out_radius_gas * scale)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset)) * out_radius_gas * scale,
                          center_y * scale - math.cos(math.radians(i + offset)) * out_radius_gas * scale)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset)) * in_radius_gas * scale,
                          center_y * scale - math.cos(math.radians(i + offset)) * in_radius_gas * scale)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset + 5)) * in_radius_gas * scale,
                          center_y * scale - math.cos(math.radians(i + offset + 5)) * in_radius_gas * scale)
            ac.glEnd()

        ac.glBegin(1)
        for i in range(0, degreeBrake, 5):
            ac.glBegin(3)
            ac.glColor4f(1, 0, 0, 0.75)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset + 5)) * out_radius_brake * scale,
                          center_y * scale - math.cos(math.radians(i + offset + 5)) * out_radius_brake * scale)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset)) * out_radius_brake * scale,
                          center_y * scale - math.cos(math.radians(i + offset)) * out_radius_brake * scale)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset)) * in_radius_brake * scale,
                          center_y * scale - math.cos(math.radians(i + offset)) * in_radius_brake * scale)
            ac.glVertex2f(center_x * scale + math.sin(math.radians(i + offset + 5)) * in_radius_brake * scale,
                          center_y * scale - math.cos(math.radians(i + offset + 5)) * in_radius_brake * scale)
            ac.glEnd()

    if theme != 5:
        if info.graphics.status != 1:
            if show_drift:
                if 30 > angle_car >= 15 and speed > 5:
                    ac.glColor4f(1, 1, 1, alpha)
                    ac.glQuadTextured(drift_x * scale, drift_y * scale, drift_width * scale, drift_height * scale,
                                      drift_blue)
                elif angle_car >= 30 and speed > 5:
                    ac.glColor4f(1, 1, 1, 1)
                    ac.glQuadTextured(drift_x * scale, drift_y * scale, drift_width * scale, drift_height * scale,
                                      drift_yellow)

    if theme == 7:
        if rpm > maxrpmred:
            ac.glColor4f(1, 1, 1, 1)
            ac.glQuadTextured(12 * scale, 12 * scale, 280 * scale, 280 * scale, rev_light)
        elif maxrpmred > rpm > maxrpmbrown:
            ac.glColor4f(1, 1, 1, 1)
            ac.glQuadTextured(12 * scale, 12 * scale, 280 * scale, 280 * scale, rev_brown)
		


# Settings window
def settings_window_activated(*args):
    global settings_window_visibility

    settings_window_visibility = 1


def settings_window_deactivated(*args):
    global settings_window_visibility

    settings_window_visibility = 0


# Settings window listeners
def theme_spinner_clicked(*args):
    global config, update_config

    update_config = True

    config.set("IDAS6hud", "theme", "{:.0f}".format(ac.getValue(theme_spinner)))


def scale_spinner_clicked(*args):
    global config, update_config

    update_config = True

    config.set("IDAS6hud", "scale", str(ac.getValue(scale_spinner)))


def unit_kmh_checkbox_clicked(*args):
    global config, update_config, unit_kmh

    update_config = True

    if unit_kmh:
        unit_kmh = False
        ac.setBackgroundTexture(speed_unit, mph)
    else:
        unit_kmh = True
        ac.setBackgroundTexture(speed_unit, kmh)

    config.set("IDAS6hud", "unit_kmh", str(unit_kmh))


def show_drift_checkbox_clicked(*args):
    global config, update_config, show_drift

    update_config = True

    if show_drift:
        show_drift = False
        if show_pedal:
            ac.setBackgroundTexture(app_window, nodrift_background_pedal)
        else:
            ac.setBackgroundTexture(app_window, nodrift_background)
    else:
        show_drift = True
        if show_pedal:
            ac.setBackgroundTexture(app_window, background_pedal)
        else:
            ac.setBackgroundTexture(app_window, background)

    config.set("IDAS6hud", "show_drift", str(show_drift))


def show_pedal_checkbox_clicked(*args):
    global config, update_config, show_pedal

    update_config = True

    if show_pedal:
        show_pedal = False
        ac.setSize(app_window, tacho_width * scale, tacho_height * scale)
        if show_drift:
            ac.setBackgroundTexture(app_window, background)
        else:
            ac.setBackgroundTexture(app_window, nodrift_background)

    else:
        show_pedal = True
        ac.setSize(app_window, tachowithPedal_width * scale, tachowithPedal_height * scale)
        if show_drift:
            ac.setBackgroundTexture(app_window, background_pedal)
        else:
            ac.setBackgroundTexture(app_window, nodrift_background_pedal)

    config.set("IDAS6hud", "show_pedal", str(show_pedal))


def night_checkbox_clicked(*args):
    global config, update_config, night_mode, gauge_path, rpm_path

    update_config = True

    if night_mode:
        night_mode = False
        gauge_path = theme_path + "background/labels_"
        rpm_path = theme_path + "rpm_bar/rpm_"
    else:
        night_mode = True
        gauge_path = theme_path + "background/night_labels_"
        rpm_path = theme_path + "rpm_bar/night_rpm_"

    config.set("IDAS6hud", "night_mode", str(night_mode))


def refresh_rate_checkbox_clicked(*args):
    global config, update_config, lower_refresh_rate

    update_config = True

    if lower_refresh_rate:
        lower_refresh_rate = False
    else:
        lower_refresh_rate = True

    config.set("IDAS6hud", "lower_refresh_rate", str(lower_refresh_rate))


def unlimited_name_checkbox_clicked(*args):
    global config, update_config, unlimited_name

    update_config = True

    if unlimited_name:
        unlimited_name = False
    else:
        unlimited_name = True

    config.set("IDAS6hud", "unlimited_name", str(unlimited_name))


def acUpdate(deltaT):
    global timer, timer2, timer3, flash_rate, alpha
    global current_car, lap_value, maxRpm, spin_rate, speed_path, maxrpmred, maxrpmbrown
    global rpm, speed, gas, brake
    global lapTime, lapTime_seconds, lapTime_minutes
    global x1, y1, z1, x2, y2, z2, distance
    global bestLap, bestLap_seconds, bestLap_minutes
    global lastLap, lastLap_seconds, lastLap_minutes
    global angle_fl, angle_fr, angle_rl, angle_rr, angle_car
    global driverName, carName, session_type, position

    # Update the timers
    timer += deltaT
    timer2 += deltaT

    if alpha < 0:
        flash_rate = 7.2
    if alpha > 1.2:
        flash_rate = -7.2

    if theme != 5:
        if info.graphics.status != 1:
            if show_drift:
                alpha += (deltaT * flash_rate)

    if lower_refresh_rate:
        timer3 += deltaT

    # Fetch data 10 times per second
    if timer > 0.1:
        timer = 0

        current_car = ac.getFocusedCar()
        lap_value = ac.getCarState(current_car, acsys.CS.LapCount) + 1
        maxRpm = info.static.maxRpm
        maxrpmred = maxRpm - 600
        maxrpmbrown = maxRpm - 1000
		
        if show_drift and info.graphics.status != 1 and theme != 5:
            angle_fl, angle_fr, angle_rl, angle_rr = ac.getCarState(current_car, acsys.CS.SlipAngle)
            angle_car = abs((angle_rl + angle_rr) / 2)

        if ac.getCarName(current_car) in ["ks_toyota_ae86", "ks_toyota_ae86_drift", "ks_toyota_ae86_tuned"]:
            spin_rate = 33.3
        else:
            if maxRpm > 17000:
                spin_rate = 83.3
            elif maxRpm > 15000:
                spin_rate = 75
            elif maxRpm > 13000:
                spin_rate = 66.7
            elif maxRpm > 10000:
                spin_rate = 54.2
            elif maxRpm > 8500:
                spin_rate = 41.7
            elif maxRpm > 7500:
                spin_rate = 37.5
            elif maxRpm > 500:
                spin_rate = 33.3
            else:
                spin_rate = 41.7

    # Calculate 30 times per second or as fast as the frame rate
    if lower_refresh_rate:
        if timer3 > 0.0333:
            timer3 = 0
            # update_rpm_gauge()
            rpm = ac.getCarState(current_car, acsys.CS.RPM)
            ac.setBackgroundTexture(rpm_gauge, app_path + rpm_path + "{:.0f}".format(rpm / spin_rate).zfill(3) + ".png")

            if show_pedal:
                gas = ac.getCarState(current_car, acsys.CS.Gas)
                brake = ac.getCarState(current_car, acsys.CS.Brake)

    else:
        # update_rpm_gauge()
        rpm = ac.getCarState(current_car, acsys.CS.RPM)
        ac.setBackgroundTexture(rpm_gauge, app_path + rpm_path + "{:.0f}".format(rpm / spin_rate).zfill(3) + ".png")

        # update_speed_digits()
        if unit_kmh:
            speed = ac.getCarState(current_car, acsys.CS.SpeedKMH)
        else:
            speed = ac.getCarState(current_car, acsys.CS.SpeedMPH)

        if theme == 7:
            if speed < 1000:
                speed_path = theme_path + "speed_blue/speed_digits_"
            elif 1000 <= speed < 1490:
                speed_path = theme_path + "speed_blue/speed_digits_"
            else:
                speed_path = theme_path + "speed_blue/speed_digits_"

        for i, label in enumerate(speed_digits):
            ac.setBackgroundTexture(speed_digits[i],
                                    app_path + speed_path + list("{:.0f}".format(speed).rjust(3, 'z'))[i] + ".png")

        if show_pedal:
            gas = ac.getCarState(current_car, acsys.CS.Gas)
            brake = ac.getCarState(current_car, acsys.CS.Brake)

        # update_current_lap()
        lapTime = ac.getCarState(current_car, acsys.CS.LapTime)
        lapTime_seconds = (lapTime / 1000) % 60
        lapTime_minutes = (lapTime // 1000) // 60
        if theme == 5:
            ac.setText(current_lap, "{:02.0f}:{:06.3f}".format(lapTime_minutes, lapTime_seconds))
        else:
            ac.setText(current_lap, "{:.0f}'{:06.3f}".format(lapTime_minutes, lapTime_seconds))

    # Calculate 10 times per second
    if timer2 > 0.1:
        timer2 = 0

        # update_lap_count()
        ac.setText(lap_count, str(lap_value))

        # update_gauge_label()
        if ac.getCarName(current_car) in ["ks_toyota_ae86", "ks_toyota_ae86_drift", "ks_toyota_ae86_tuned"]:
            ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "86.png")
        else:
            if maxRpm > 17000:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "20k.png")
            elif maxRpm > 15000:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "18k.png")
            elif maxRpm > 13000:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "16k.png")
            elif maxRpm > 10000:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "13k.png")
            elif maxRpm > 8500:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "10k.png")
            elif maxRpm > 7500:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "9k.png")
            elif maxRpm > 500:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "8k.png")
            else:
                ac.setBackgroundTexture(gauge_label, app_path + gauge_path + "10k.png")

        if lower_refresh_rate:
            # update_speed_digits()
            if unit_kmh:
                speed = ac.getCarState(current_car, acsys.CS.SpeedKMH)
            else:
                speed = ac.getCarState(current_car, acsys.CS.SpeedMPH)

            if theme == 7:
                if speed < 1000:
                    speed_path = theme_path + "speed_blue/speed_digits_"
                elif 1000 <= speed < 1490:
                    speed_path = theme_path + "speed_blue/speed_digits_"
                else:
                    speed_path = theme_path + "speed_blue/speed_digits_"

            for i, label in enumerate(speed_digits):
                ac.setBackgroundTexture(speed_digits[i],
                                        app_path + speed_path + list("{:.0f}".format(speed).rjust(3, 'z'))[i] + ".png")

            # update_current_lap()
            lapTime = ac.getCarState(current_car, acsys.CS.LapTime)
            lapTime_seconds = (lapTime / 1000) % 60
            lapTime_minutes = (lapTime // 1000) // 60
            if theme == 5:
                ac.setText(current_lap, "{:02.0f}:{:06.3f}".format(lapTime_minutes, lapTime_seconds)[:-2])
            else:
                ac.setText(current_lap, "{:.0f}'{:06.3f}".format(lapTime_minutes, lapTime_seconds)[:-2])

        # update_gear_label()
        ac.setBackgroundTexture(gear_label, app_path + theme_path + "gears/gear_" + str(
            ac.getCarState(current_car, acsys.CS.Gear)) + ".png")

        # update_driver_label()
        if unlimited_name:
            driverName = ac.getDriverName(current_car)
        else:
            driverName = '{:.14}'.format(ac.getDriverName(current_car))
        ac.setText(driver_label, driverName)

        # update_car_label()
        carName = ac.getCarName(current_car).replace('_', ' ')
        ac.setText(car_label, str.upper(carName))

        # update_position_digits()
        session_type = info.graphics.session
        if info.graphics.status == 1:
            position = str('-z')
        elif session_type == 2:
            position = str(ac.getCarRealTimeLeaderboardPosition(current_car) + 1).ljust(2, 'z')
        else:
            position = str(ac.getCarLeaderboardPosition(current_car)).ljust(2, 'z')
        for i, label in enumerate(position_digits):
            ac.setBackgroundTexture(position_digits[i],
                                    app_path + theme_path + "position_digits/position_digits_" + list(position)[
                                        i] + ".png")

        # update_best_lap()
        if lap_value == 1:
            if theme == 5:
                ac.setText(best_lap, "––:––.–––")
            else:
                ac.setText(best_lap, "–'––.–––")
        else:
            bestLap = ac.getCarState(current_car, acsys.CS.BestLap)
            bestLap_seconds = (bestLap / 1000) % 60
            bestLap_minutes = (bestLap // 1000) // 60
            if theme == 5:
                ac.setText(best_lap, "{:02.0f}:{:06.3f}".format(bestLap_minutes, bestLap_seconds))
            else:
                ac.setText(best_lap, "{:.0f}'{:06.3f}".format(bestLap_minutes, bestLap_seconds))

        # update_last_lap()
        if lap_value == 1:
            if theme == 5:
                ac.setText(last_lap, "––:––.–––")
            else:
                ac.setText(last_lap, "–'––.–––")
        else:
            lastLap = ac.getCarState(current_car, acsys.CS.LastLap)
            lastLap_seconds = (lastLap / 1000) % 60
            lastLap_minutes = (lastLap // 1000) // 60
            if theme == 5:
                ac.setText(last_lap, "{:02.0f}:{:06.3f}".format(lastLap_minutes, lastLap_seconds))
            else:
                ac.setText(last_lap, "{:.0f}'{:06.3f}".format(lastLap_minutes, lastLap_seconds))

        # Opacity needs to be reset if the app was moved
        ac.setBackgroundOpacity(app_window, 0)
        ac.setBackgroundOpacity(map_window, 0)
        ac.setBackgroundOpacity(time_window, 0)
        ac.setBackgroundOpacity(record_window, 0)
        ac.setBackgroundOpacity(name_window, 0)


def acShutdown():
    global config, update_config, config_path

    # Update config if needed
    if update_config:
        with open(config_path, 'w') as file_config:
            config.write(file_config)
